/*
 * media-test - Simple utility for testing various memory media
 * Copyright (C) 2004 Ales Hakl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "output.h"
#include "tester.h"

#include <stdio.h>
#include <malloc.h>

output_context_t* stdout_init(){
  output_context_t* ctx;

  //    0123456789abcdef FAIL 0123456789 Errors
  ctx = malloc(sizeof(output_context_t));

  ctx->sequential_block=&stdout_sequential_block;
  ctx->sequential_head=&stdout_sequential_head;
  ctx->random_block=&stdout_random_block;
  ctx->random_head=&stdout_random_head;
  ctx->done=&stdout_done;

  return ctx;

}
int stdout_sequential_head(void*pdata,
			   size_t block_size, 
			   char* reader_name, 
			   char* reader_params){
  printf("Block-Size: 0x%x\n",block_size);
  printf("Reader: %s\n",reader_name);
  printf("Reader-Params: %s\n\n",reader_params);

  puts("    Start offset  Stat  Read time  Errors");
  puts("=========================================");
}
int stdout_random_head(void*pdata,
		       size_t block_size, 
		       int64_t iterations, 
		       char* reader_name, 
		       char* reader_params){
  printf("Block-Size: 0x%x\n",block_size);
  printf("Iterations: %lld\n",iterations);
  printf("Reader: %s\n",reader_name);
  printf("Reader-Params: %s\n\n",reader_params);

  puts("    Start offset     Seek distance  Stat  Read time  Errors");
  puts("===========================================================");
}
int stdout_done(output_context_t* ctx){
  free(ctx);
}
int stdout_sequential_block(void*pdata, off_t start, int64_t time, int state, int errors){
  if (errors==T_NA){
    if (state==T_OK){
      //      0123456789abcdef FAIL 0123456789
      printf(          "%16llx   OK  %10lld\n",start,time);
    }else{
      //      0123456789abcdef FAIL 0123456789
      printf(          "%16llx  FAIL %10lld\n",start,time);
      
    }
  }else{
    if (state==T_OK){
      //      0123456789abcdef FAIL 0123456789
      printf(          "%16llx   OK  %10lld  %d\n",start,time,errors);
    }else{
      //      0123456789abcdef FAIL 0123456789
      printf(          "%16llx  FAIL %10lld  %d\n",start,time,errors);
      
    }

  }
}
int stdout_random_block(void*pdata, off_t start, int64_t time, int state, int errors, off_t dist){
  if (errors==T_NA){
    if (state==T_OK){
      //      0123456789abcdef FAIL 0123456789
      printf(          "%16llx %16llx   OK  %10lld\n",start,dist,time);
    }else{
      //      0123456789abcdef FAIL 0123456789
      printf(          "%16llx %16llx  FAIL %10lld\n",start,dist,time);
      
    }
  }else{
    if (state==T_OK){
      //      0123456789abcdef 0123456789abcdef FAIL 0123456789
      printf(          "%16llx %16llx   OK  %10lld  %d\n",start,dist,time,errors);
    }else{
      //      0123456789abcdef 0123456789abcdef FAIL 0123456789
      printf(          "%16llx %16llx  FAIL %10lld  %d\n",start,dist,time,errors);
      
    }

  }
}

output_context_t* xml_init(){
  output_context_t* ctx;

  //    0123456789abcdef FAIL 0123456789 Errors
  ctx = malloc(sizeof(output_context_t));

  ctx->sequential_block=&xml_sequential_block;
  ctx->sequential_head=&xml_sequential_head;
  ctx->random_block=&xml_random_block;
  ctx->random_head=&xml_random_head;
  ctx->done=&xml_done;

  return ctx;

}

static char* xml_escape_string(char* str){
  int i,j;
  size_t len;
  char *tmp;

  len = strlen(str)+64;
  tmp = malloc (len);
  if (!tmp){
    return NULL;
  }
  for (i=0, j=0; j<(len-6) && str[i]!=0; i++){
    switch(str[i]){
    case '&':
      tmp[j]='&';
      tmp[j+1]='a';
      tmp[j+2]='m';
      tmp[j+3]='p';
      tmp[j+4]=';';
      j+=5;
      break;
    case '<':
      tmp[j]='&';
      tmp[j+1]='l';
      tmp[j+2]='t';
      tmp[j+3]=';';
      j+=4;
      break;
    default:
      tmp[j]=str[i];
      ++j;
    }
  }
  tmp[j]=0;
  return tmp;
}

int xml_sequential_head(void*pdata,size_t block_size, char* reader_name, char* reader_params){
  char * reader_name_esc = xml_escape_string(reader_name);
  char * reader_params_esc = xml_escape_string(reader_params);


  puts("<?xml version=\"1.0\" ?>");
  puts("<media-test mode=\"linear\">");

  puts("  <header>");
  printf("    <block-size>%d</block-size>\n",block_size);
  printf("    <reader>%s</reader>\n",reader_name_esc);
  printf("    <reader-params>%s</reader-params>\n",reader_params_esc);
  puts("  </header>");

  free(reader_name_esc);
  free(reader_params_esc);

}
int xml_random_head(void*pdata,size_t block_size, int64_t iterations, char* reader_name, char* reader_params){

  char * reader_name_esc = xml_escape_string(reader_name);
  char * reader_params_esc = xml_escape_string(reader_params);

  puts("<?xml version=\"1.0\" ?>");
  puts("<media-test mode=\"random\">");
  puts("  <header>");
  printf("    <block-size>%d</block-size>\n",block_size);
  printf("    <iterations> %lld</iterations>\n",iterations);
  printf("    <reader>%s</reader>\n",reader_name_esc);
  printf("    <reader-params>%s</reader-params>\n",reader_params_esc);
  puts("  </header>");

  free(reader_name_esc);
  free(reader_params_esc);

}
int xml_done(output_context_t* ctx){
  puts("</media-test>");
  free(ctx);
}
int xml_sequential_block(void*pdata, off_t start, int64_t time, int state, int errors){

  char strtab[3][6]={"ERROR","OK","FAIL"};

  printf("  <block>\n    <start>%lld</start>\n    <time>%lld</time>\n    <state>%s</state>\n    <errors>%d</errors>\n  </block>\n",start,time,strtab[state],errors);
}
int xml_random_block(void*pdata, off_t start, int64_t time, int state, int errors, off_t dist){

  char strtab[3][6]={"ERROR","OK","FAIL"};

  printf("  <block>\n    <start>%lld</start>\n    <time>%lld</time>\n    <state>%s</state>\n    <errors>%d</errors>\n    <distance>%lld</distance>\n  </block>\n",start,time,strtab[state],errors,dist);
}

#if 0
typedef struct gnuplot_pdata_t{
  FILE* time;
  FILE* errors;
  FILE* state;
} gnuplot_pdata_t;

output_context_t* gnuplot_init(char* base_name){
  output_context_t* ctx;
  char *time,*errors,*state,*main;
  gnuplot_pdata_t* pd;
  FILE *f_main;

  time=malloc(strlen(base_name)+10);
  errors=malloc(strlen(base_name)+10);
  state=malloc(strlen(base_name)+10);
  main=malloc(strlen(base_name)+10);

  strncpy(time,base_name,strlen(base_name)+10);
  strncpy(errors,base_name,strlen(base_name)+10);
  strncpy(state,base_name,strlen(base_name)+10);
  strncpy(main,base_name,strlen(base_name)+10);
  strcat(time,".time");
  strcat(errors,".errors");
  strcat(state,".state");
  strcat(main,".gnuplot");
  
  f_main = fopen(main,"w");
  if (f_main==NULL)
    return NULL;
  fprintf(f_main,"set autoscale\nplot '%s' '%s' '%s'\n",time,errors,state);
  fclose(f_main);

  pd = malloc(sizeof(gnuplot_pdata_t));

  if (!pd)
    return NULL;

  pd->time = fopen(time,"w");
  if (pd->time==NULL){
    free(pd);
    return NULL;
  }
  pd->errors = fopen(errors,"w");
  if (pd->errors==NULL){
    fclose(pd->time);
    free(pd);
    return NULL;
  }
  pd->state = fopen(state,"w");
  if (pd->state==NULL){
    fclose(pd->time);
    fclose(pd->errors);
    free(pd);
    return NULL;
  }

  //    0123456789abcdef FAIL 0123456789 Errors
  ctx = malloc(sizeof(output_context_t));

  ctx->linear_block=&gnuplot_block;
  ctx->linear_head=&gnuplot_head;
  ctx->done=&gnuplot_done;
  ctx->pdata=(void*)pd;

  return ctx;

}
int gnuplot_head(void*pdata){
}
int gnuplot_done(output_context_t*ctx){
    fclose(((gnuplot_pdata_t*)ctx->pdata)->time);
    fclose(((gnuplot_pdata_t*)ctx->pdata)->errors);
    fclose(((gnuplot_pdata_t*)ctx->pdata)->state);
    free(ctx->pdata);
    free(ctx);
}
int gnuplot_block(void*pdata, off_t start, int64_t time, int state, int errors){
  if (state==T_OK){
    //      0123456789abcdef FAIL 0123456789
    fprintf(((gnuplot_pdata_t*)pdata)->time,"%lld %lld\n",start,time);
  }else{
    //      0123456789abcdef FAIL 0123456789
    fprintf(((gnuplot_pdata_t*)pdata)->time,"\n",start,time); 
  }
  fprintf(((gnuplot_pdata_t*)pdata)->errors,"%lld %d\n",start,errors);
  fprintf(((gnuplot_pdata_t*)pdata)->state,"%lld %d\n",start,state);

}
#endif
