/*
 * media-test - Simple utility for testing various memory media
 * Copyright (C) 2004 Ales Hakl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef H_output__
#define H_output__

#include <sys/types.h>


#define O_ERROR 0
#define O_OK 1


typedef struct output_context_t{
  void*pdata;
  int (*sequential_block)(void*pdata, 
			  off_t start, 
			  int64_t time, 
			  int state, 
			  int errors);
  int (*sequential_head)(void*pdata, 
		     size_t block_size, 
		     char* reader, 
		     char* reader_params);
  int (*random_block)(void*pdata, 
		      off_t start, 
		      int64_t time, 
		      int state, 
		      int errors, 
		      off_t dist);
  int (*random_head)(void*pdata,
		     size_t block_size,
		     int64_t iterations,
		     char* reader,
		     char* reader_params);
  int (*done)(struct output_context_t*ctx);
}output_context_t;

#define output_done(context) ((*(context)->done)((context)))
#define output_sequential_head(context,block_size,reader_name,reader_params) ((*(context)->sequential_head)((context)->pdata,(block_size),(reader_name),(reader_params)))
#define output_sequential_block(context,start,time,state,errors) ((*(context)->sequential_block)(((context)->pdata),(start),(time),(state),(errors)))
#define output_random_head(context,block_size,iterations,redear_name,reader_params) ((*(context)->random_head)((context)->pdata,(block_size),(iterations),(reader_name),(reader_params)))
#define output_random_block(context,start,time,state,errors,dist) ((*(context)->random_block)(((context)->pdata),(start),(time),(state),(errors),(dist)))

extern output_context_t* stdout_init();
extern int stdout_done(output_context_t* ctx);
extern int stdout_sequential_head(void*pdata,
			      size_t block_size, 
			      char* reader_name, 
			      char* reader_params);
extern int stdout_sequential_block(void*pdata, 
			       off_t start, 
			       int64_t time, 
			       int state, 
			       int errors);
extern int stdout_random_head(void*pdata,
			      size_t block_size, 
			      int64_t iterations, 
			      char* reader_name, 
			      char* reader_params);
extern int stdout_random_block(void*pdata, 
			       off_t start, 
			       int64_t time, 
			       int state, 
			       int errors, 
			       off_t dist);

extern output_context_t* xml_init();
extern int xml_done(output_context_t* ctx);
extern int xml_sequential_head(void*pdata,
			   size_t block_size, 
			   char* reader_name, 
			   char* reader_params);
extern int xml_sequential_block(void*pdata, 
			    off_t start, 
			    int64_t time, 
			    int state, 
			    int errors);
extern int xml_random_head(void*pdata,
			   size_t block_size, 
			   int64_t iterations, 
			   char* reader_name, 
			   char* reader_params);
extern int xml_random_block(void*pdata, 
			    off_t start, 
			    int64_t time, 
			    int state, 
			    int errors, 
			    off_t dist);




#endif
