/*
 * media-test - Simple utility for testing various memory media
 * Copyright (C) 2004 Ales Hakl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef H_tester__
#define H_tester__

#include <sys/types.h>

#define T_OK 1      // OK
#define T_ERROR 0   // Internal error in testing mechanism
#define T_FAIL 2    // Error on media
#define T_NA -1

typedef struct test_context_t {
  void * pdata;
  int (*init)(void **pdata, char*prm);
  int (*done)(void *pdata);
  int (*alloc)(void *pdata, size_t length);
  int (*free)(void *pdata);
  off_t (*blast)(void *pdata);
  int (*bread)(void *pdata, off_t start);
  int (*errcnt)(void *pdata);
  int (*ecreset)(void *pdata);
} test_context_t;

#define test_init(context,prm) ((*(context)->init)(&((context)->pdata),(prm)))
#define test_done(context) ((*(context)->done)((context)->pdata))
#define test_alloc(context,length) ((*(context)->alloc)((context)->pdata,(length)))
#define test_free(context) ((*(context)->free)((context)->pdata))
#define test_blast(context) ((*(context)->blast)((context)->pdata))
#define test_bread(context,start) ((*(context)->bread)((context)->pdata,(start)))
#define test_errcnt(context) ((context)->errcnt?(*(context)->errcnt)((context)->pdata):T_NA)
#define test_ecreset(context) ((context)->ecreset?(*(context)->ecreset)((context)->pdata):T_NA)
#define test_ecavail(context) ((context)->errcnt && (context)->ecreset)

extern int time_bread(test_context_t *context, off_t start, int64_t* time);

#endif
