/*
 * media-test - Simple utility for testing various memory media
 * Copyright (C) 2004 Ales Hakl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "unix.h"

#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>


typedef struct unix_pdata_t {
  void*buf;
  size_t length;
  int fd;
} unix_pdata_t;

void unix_fill_context(test_context_t* context){
  context->init=&unix_init;
  context->done=&unix_done;
  context->alloc=&unix_alloc;
  context->free=&unix_free;
  context->bread=&unix_bread;
  context->blast=&unix_blast;
  context->errcnt=NULL;
  context->ecreset=NULL;

}
void unix_fill_context_direct(test_context_t* context){
  context->init=&unix_init_direct;
  context->done=&unix_done;
  context->alloc=&unix_alloc_direct;
  context->free=&unix_free;
  context->bread=&unix_bread;
  context->blast=&unix_blast;
  context->errcnt=NULL;
  context->ecreset=NULL;

}

int unix_init(void**pdata, char* prm){
  if (!pdata)
    return T_ERROR;

  *pdata = malloc(sizeof(unix_pdata_t));
  if (!*pdata)
    return T_ERROR;

  ((unix_pdata_t*)*pdata)->fd = open(prm,O_RDONLY);

  if (-1==((unix_pdata_t*)*pdata)->fd){
    return T_ERROR;
  }

  ((unix_pdata_t*)*pdata)->length=0;
  ((unix_pdata_t*)*pdata)->buf=NULL;

  return T_OK;


}
int unix_init_direct(void**pdata, char* prm){


  if (!pdata)
    return T_ERROR;

  *pdata = malloc(sizeof(unix_pdata_t));
  if (!*pdata)
    return T_ERROR;

#ifndef O_DIRECT
  fputs("warning: O_DIRECT not supported, using O_RSYNC instead.");
  ((unix_pdata_t*)*pdata)->fd = open(prm,O_RDONLY | O_RSYNC);
#else
  ((unix_pdata_t*)*pdata)->fd = open(prm,O_RDONLY | O_DIRECT);
#endif

  if (-1==((unix_pdata_t*)*pdata)->fd){
    return T_ERROR;
  }

  ((unix_pdata_t*)*pdata)->length=0;
  ((unix_pdata_t*)*pdata)->buf=NULL;

  return T_OK;


}
int unix_done(void*pdata){
  if (!pdata)
    return T_ERROR;

  if (((unix_pdata_t*)pdata)->length!=0){
    unix_free(pdata);
  }

  free(pdata);
  return T_OK;

}
int unix_alloc(void*pdata, size_t length){
  if (!pdata)
    return T_ERROR;

  if (((unix_pdata_t*)pdata)->length!=0){
    unix_free(pdata);
  }

  ((unix_pdata_t*)pdata)->buf = malloc(length);

  if (!((unix_pdata_t*)pdata)->buf)
    return T_ERROR;

  ((unix_pdata_t*)pdata)->length=length;

  return T_OK;

}
int unix_alloc_direct(void*pdata, size_t length){
  if (!pdata)
    return T_ERROR;

  if (((unix_pdata_t*)pdata)->length!=0){
    unix_free(pdata);
  }

  posix_memalign((&((unix_pdata_t*)pdata)->buf),length,length);

  if (!((unix_pdata_t*)pdata)->buf)
    return T_ERROR;

  ((unix_pdata_t*)pdata)->length=length;

  return T_OK;

}
int unix_free(void*pdata){
  if (!pdata)
    return T_ERROR;

  free (((unix_pdata_t*)pdata)->buf);

  ((unix_pdata_t*)pdata)->length=0;

  return T_OK;

}
off_t unix_blast(void*pdata){
  off_t last;
  if (!pdata)
    return T_ERROR;
  
  last=lseek(((unix_pdata_t*)pdata)->fd,0,SEEK_END);
  
  return last;
}
int unix_bread(void*pdata, off_t start){
  if (!pdata)
    return T_ERROR;
  if (((unix_pdata_t*)pdata)->length==0)
    return T_ERROR;

  if ((off_t)-1==lseek(((unix_pdata_t*)pdata)->fd,start,SEEK_SET))
    return T_FAIL;
  if ((ssize_t)-1==read(((unix_pdata_t*)pdata)->fd,((unix_pdata_t*)pdata)->buf,
	       ((unix_pdata_t*)pdata)->length))
    return T_FAIL;

  return T_OK;
 
}


// from tester.h:

int time_bread(test_context_t *context, off_t start, int64_t* time){
  struct timeval tv;
  int64_t t_st,t_sp;
  int r;
  gettimeofday(&tv,NULL);
  t_st=tv.tv_usec+tv.tv_sec*1000000;
  r = test_bread(context,start);
  gettimeofday(&tv,NULL);
  t_sp=tv.tv_usec+tv.tv_sec*1000000;
  *time=t_sp-t_st;
  return r;
}
